/*
 * Copyright (c) 2013, Pushkar Singh <psingh@sangoma.com>
 * Sangoma Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * * Neither the name of the original author; nor the names of any contributors
 *   may be used to endorse or promote products derived from this software
 *   without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * Contributors: Pushkar Singh <psingh@sangoma.com>
 */

#ifndef __SNG_ISDN_TAP_H__
#define __SNG_ISDN_TAP_H__

/********************************************************************************/
/*                                                                              */
/*                         Structure Declearation                               */
/*                                                                              */
/********************************************************************************/
/* D channel controller structure */
struct sngisdn_t {
	ftdm_socket_t fd;		/* File descriptor for D-Channel */
	void *userdata;
	int debug;                      /* Debug stuff */
	int state;                      /* State of D-channel */
	int switchtype;         	/* Switch type */
	int localtype;			/* local network type */
	int cref;                       /* Next call reference value */
} sngisdn_t;

typedef enum {
	ISDN_TAP_RUNNING = (1 << 0),  
	ISDN_TAP_MASTER = (1 << 1),  
} sngisdn_tap_flags_t;

typedef enum {
	ISDN_TAP_MIX_BOTH = 0, 
	ISDN_TAP_MIX_PEER,
	ISDN_TAP_MIX_SELF,
} sngisdn_tap_mix_mode_t;

typedef struct {
	ftdm_number_t callingnum;
	ftdm_number_t callednum;
	ftdm_channel_t *fchan;
	uint8_t proceeding;
	int inuse;
	int callref_val;
	int chanId;
} passive_call_t;

typedef enum sngisdn_tap_iface {
	ISDN_TAP_IFACE_UNKNOWN = 0,
	ISDN_TAP_IFACE_CPE = 1,
	ISDN_TAP_IFACE_NET = 2,
} sngisdn_tap_iface_t;

typedef struct isdn_tap {
	int32_t flags;
	struct sngisdn_t *pri;
	int debug;
	int switch_type;
	sngisdn_tap_mix_mode_t mixaudio;
	ftdm_channel_t *dchan;
	ftdm_span_t *span;
	ftdm_span_t *peerspan;
	ftdm_mutex_t *pcalls_lock;
	passive_call_t pcalls[FTDM_MAX_CHANNELS_PHYSICAL_SPAN];
	sngisdn_tap_iface_t iface;
} sngisdn_tap_t;

/********************************************************************************/
/*                                                                              */
/*                               Function Prototype                             */
/*                                                                              */
/********************************************************************************/
void *sangoma_isdn_get_userdata(struct sngisdn_t *pri);

void sangoma_isdn_set_message(void (*__pri_error)(struct sngisdn_t *pri, char *));
void sangoma_isdn_set_error(void (*__pri_error)(struct sngisdn_t *pri, char *));

struct sngisdn_t *sangoma_isdn_new_cb(ftdm_socket_t fd, int nodetype, int switchtype, void *userdata);

/********************************************************************************/
/*                                                                              */
/*                                     MACROS                                   */
/*                                                                              */
/********************************************************************************/
/* Node Types */
#define PRI_NETWORK             1
#define PRI_CPE                 2

#define SNG_ISDN_SPAN(p) (((p) >> 8) & 0xff)
#define SNG_ISDN_CHANNEL(p) ((p) & 0xff)

#define ISDN_TAP_NETWORK_ANSWER 0x1

/* Debugging Levels */
#define DEBUG_Q921_RAW              (1 << 0)        /* Show raw HDLC frames */
#define DEBUG_ALL                   (0xffff)        /* Everything */

/* Transmit capabilities */
#define TRANS_CAP_SPEECH            	0x0
#define TRANS_CAP_DIGITAL           	0x08
#define TRANS_CAP_RESTRICTED_DIGITAL        0x09
#define TRANS_CAP_3_1K_AUDIO                0x10
#define TRANS_CAP_7K_AUDIO                  0x11    /* Depriciated ITU Q.931 (05/1998)*/
#define TRANS_CAP_DIGITAL_W_TONES           0x11
#define TRANS_CAP_VIDEO                     0x18

#define LAYER_1_ITU_RATE_ADAPT      	0x21
#define LAYER_1_ULAW                        0x22
#define LAYER_1_ALAW                        0x23
#define LAYER_1_G721                        0x24
#define LAYER_1_G722_G725           	0x25
#define LAYER_1_H223_H245           	0x26
#define LAYER_1_NON_ITU_ADAPT       	0x27
#define LAYER_1_V120_RATE_ADAPT     	0x28
#define LAYER_1_X31_RATE_ADAPT      	0x29

/* ISDN MSGTYPE DECODE **********************************************************/
#define SNG_DECODE_ISDN_EVENT(msgType) \
((msgType) == Q931_ALERTING)        ?"ALERT" : \
((msgType) == Q931_PROCEEDING)      ?"CALL PROCEED" : \
((msgType) == Q931_PROGRESS)        ?"PROGRESS" : \
((msgType) == Q931_SETUP)           ?"SETUP" : \
((msgType) == Q931_CONNECT)         ?"CONNECT" : \
((msgType) == Q931_SETUP_ACK)       ?"SETUP ACK" : \
((msgType) == Q931_CONNECT_ACK)     ?"CONNECT ACK" : \
((msgType) == Q931_USER_INFO)       ?"USER INFO" : \
((msgType) == Q931_SUSPEND_REJ)     ?"SUSPEND REJECT" : \
((msgType) == Q931_RESUME_REJ)      ?"RESUME REJECT" : \
((msgType) == Q931_SUSPEND)         ?"SUSPEND" : \
((msgType) == Q931_RESUME)          ?"RESUME" : \
((msgType) == Q931_SUSPEND_ACK)     ?"SUSPEND ACK" : \
((msgType) == Q931_RESUME_ACK)      ?"RESUME ACK" : \
((msgType) == Q931_DISCONNECT)      ?"DISCONNECT" : \
((msgType) == Q931_RESTART)         ?"RESTART" : \
((msgType) == Q931_RELEASE)         ?"RELEASE" : \
((msgType) == Q931_RESTART_ACK)     ?"RESTART ACK" : \
((msgType) == Q931_RELEASE_COMPLETE)?"RELEASE COMPLETE" : \
((msgType) == Q931_SEGMENT)         ?"SEGMENT" : \
((msgType) == Q931_FACILITY)        ?"FACILITY" : \
((msgType) == Q931_NOTIFY)          ?"NOTIFY" : \
((msgType) == Q931_STATUS_ENQUIRY)  ?"STATUS ENQUIRY" : \
((msgType) == Q931_CONGESTION_CNTRL)?"CONGESTION CONTROL" : \
((msgType) == Q931_INFORMATION)     ?"INFORMATION" : \
((msgType) == Q931_STATUS)          ?"STATUS" : \
"Unknown ISDN event"

#endif /* __SNG_ISDN_TAP_H__ */
